#include <stdio.h>
#include <stdarg.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>

#include "mac.h"
#include "dhcprogue.h"

int mac_file_init(char *file) {

	char line[1024];
	char mac[19];
	char ip[17];
	int lineno = 1;
	
	FILE *fp = fopen(file, "r");
	if(!fp)
		fatal("cannot open %s: %s", file, strerror(errno));
	else
		msg("reading MAC/IP pairs from %s", file);
	
	total_macs = 0;
	
	while(1) {
		memset(&mac, 0, sizeof(mac));
		memset(&ip,  0, sizeof(ip));
	
		if(!fgets(line, sizeof(line), fp))
			break;
		
		if(sscanf(line, "%s %s", mac, ip) != 2)
			fatal("malformed MAC/IP line @ %s:%d", file, lineno);
		
		memcpy(macs[total_macs].mac, mac, 19);
		memcpy(macs[total_macs].ip,  ip,  16);
		
		debug("%s %s added to MAC/IP pairlist", macs[total_macs].mac, macs[total_macs].ip);
		
		lineno++, total_macs++;
	}
	fclose(fp);
	
	return total_macs;
}

char * mac_get_ip(unsigned char *bin_mac) {

	unsigned int i = 0;
	char chaddr[19];
	static char ip[17];
	
	memset(&ip, 0, sizeof(ip));
	
	for(i = 0; i < _MAX_MACS; i++) {
		if(macs[i].ip[0]) {
		
			memset(chaddr, 0, sizeof(chaddr));
			snprintf(chaddr, sizeof(chaddr), "%02x:%02x:%02x:%02x:%02x:%02x", bin_mac[0], bin_mac[1], bin_mac[2], bin_mac[3], bin_mac[4], bin_mac[5]);

			debug("%s == %s?", bin_mac, chaddr);

			if(memcmp(chaddr, macs[i].mac, 18) == 0) {
				memcpy(ip, macs[i].ip, 16);
				return ip;
			}
		}
	}
	return NULL;
}

int mac_equals(char *ascii_mac, struct dhcp_packet *packet) {

	unsigned int mac1, mac2, mac3, mac4, mac5, mac6;
	
	if(sscanf(ascii_mac, "%02x:%02x:%02x:%02x:%02x:%02x", &mac1, &mac2, &mac3, &mac4, &mac5, &mac6) != 6) {
		msg("erroneous MAC address %s", ascii_mac);
		return FALSE;
	}
	if(mac1 == packet->chaddr[0] &&
	   mac2 == packet->chaddr[1] &&
	   mac3 == packet->chaddr[2] &&
	   mac4 == packet->chaddr[3] &&
	   mac5 == packet->chaddr[4] &&
	   mac6 == packet->chaddr[5]
	)
		return TRUE;
	else
		return FALSE;

}
