#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#include "dhcprogue.h"

int socket_init(int port) {

	struct sockaddr_in addr;
	int yes = 1;
	int sock = -1;

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = INADDR_ANY;	

	if((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
		fatal("socket() failed: %s", strerror(errno));
	else
		msg("socket created");
	
        if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
		fatal("setsockopt() failed to set SO_REUSEADDR: %s", strerror(errno));
	else
		msg("SO_REUSEADDR enabled");

	if(setsockopt(sock, SOL_SOCKET, SO_BROADCAST, &yes, sizeof(int)) == -1)
		fatal("setsockopt() failed to set SO_BROADCAST: %s", strerror(errno));
	else
		msg("SO_BROADCAST enabled");

	if(bind(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr)) == -1)
		fatal("bind() failed: %s", strerror(errno));
	else
		msg("listener started on %d/udp", port);

	return sock;
}

int socket_deinit(int sock) {
	msg("shutting down socket %d", sock);

	if(!shutdown(sock, SHUT_RDWR) && !close(sock))
		return TRUE;
	else
		return FALSE;
}

int socket_receive(int sock, void *buf, int buflen, struct sockaddr_in *addr) {

	unsigned int len = sizeof(struct sockaddr_in);
	unsigned int bytes = 0;

	memset(buf, 0, buflen);
	memset(addr, 0, sizeof(struct sockaddr_in));

	addr->sin_family = AF_INET;
	addr->sin_port = htons(PORT);
	addr->sin_addr.s_addr = INADDR_ANY;
	
	if((bytes = recvfrom(sock, buf, buflen, 0, (struct sockaddr *) addr, &len)) == -1)
		fatal("recvfrom() failed: %s", strerror(errno));
	else if(debugmode)
		msg("recieved packet (%d bytes) from %s", bytes, inet_ntoa(addr->sin_addr));
	
	return bytes;
}

int socket_send(int sock, void *buf, int buflen, struct sockaddr_in *addr) {

	unsigned int len = sizeof(struct sockaddr_in);
	unsigned int bytes = 0;
	
	if((bytes = sendto(sock, buf, buflen, 0, (struct sockaddr *) addr, len)) == -1)
		fatal("sendto() failed: %s", strerror(errno));
	else if(debugmode)
		msg("send packet (%d bytes) to %s", bytes, inet_ntoa(addr->sin_addr));
	
	return bytes;
}
